import kivy
kivy.require('1.0.8')

from kivy.core.window import Window
from kivy.uix.widget import Widget
from kivy.graphics import Fbo, Color, Rectangle
from kivy.app import App
from kivy.clock import Clock
import math

#PPS will stand for Pixels Per Second
pixelsPerMeter = 64

def vec2_contains(haystack_pos, haystack_size, needle_pos):
    bottomright_pos = (haystack_pos[0]+haystack_size[0], haystack_pos[1]+haystack_size[1])
    return needle_pos[0]>=haystack_pos[0] and needle_pos[0]<bottomright_pos[0] and needle_pos[1]>=haystack_pos[1] and needle_pos[1]<bottomright_pos[1]

class Character:
    color = None
    #NOTE: if Rectangle here instead of None, rectangle will be shared by all!
    rectangle = None
    hasAI = False
    visible = True
    targetX = None
    targetY = None
    maxSpeedPPS = None
    is_disposable = None
    
        

    def __init__(self):
        self.color = Color(1, 1, 1, 1)
        self.rectangle = Rectangle(pos=(0, 0), size=(64, 64))
        self.targetX = 0
        self.targetY = 0
        self.maxSpeedPPS = 10
        self.is_disposable = False

    def getDistance1D(self, value1, value2):
        result = value2 - value1
        if result < 0:
            result = result * -1
        return result

    def get_distance_vec2(self, pos1, pos2):
        # **2 means squared:
        return math.sqrt( (pos2[0] - pos1[0]) ** 2 + (pos2[1] - pos1[1]) ** 2 )

    def get_center_pos(self):
        return ( self.rectangle.pos[0] + (self.rectangle.size[0] / 2) , self.rectangle.pos[1] + (self.rectangle.size[1] / 2) )
        
    def set_center_pos(self, center_pos):
        self.rectangle.pos = (  (x - self.rectangle.size[0]/2) ,  (y - self.rectangle.size[1]/2)  )
    
    def set_target_pos(self, pos):
        targetX = pos[0]
        targetY = pos[1]

    def stop(self):
        self.targetX = self.getCenterX()
        self.targetY = self.getCenterY()

    #region to deprecate
    def getDistance2D(self,x1,y1,x2,y2):
        # **2 means squared:
        return math.sqrt( (x2 - x1) ** 2 + (y2 - y1) ** 2 )

    def getCenterX(self):
        return self.rectangle.pos[0] + (self.rectangle.size[0] / 2)

    def setCenterX(self,x):
        self.rectangle.pos = (x - self.rectangle.size[0] / 2, self.rectangle.pos[1])

    def getCenterY(self):
        return self.rectangle.pos[1] + (self.rectangle.size[1] / 2)

    def setCenterY(self,y):
        self.rectangle.pos = (self.rectangle.pos[0], (y - self.rectangle.size[1]/2))
    #endregion to deprecate



class CanvasForm(Widget):
    characters = list()
    player1 = None
    characterCount = 2

    def __init__(self, **kwargs):
        super(CanvasForm, self).__init__(**kwargs)
        self._keyboard = Window.request_keyboard(
            self._keyboard_closed, self)
        self._keyboard.bind(on_key_down=self._on_keyboard_down)
        self.bind(on_touch_down=self.canvasTouchDown)

        #NOTES:
        #pos[0] is x coordinate for position
        #pos[1] is y coordinate for position
        #The color constructor takes Red,Green,Blue, and Alpha which is opacity
        #so Color(1,0,0,1) is red

        self.characters.append(Character())
        self.characters[0].color = Color(1, 1, 1, 1)
        self.canvas.add(self.characters[0].color)
        self.canvas.add(self.characters[0].rectangle)
        self.characters[0].setCenterX(self.getWidth() / 2)
        self.characters[0].setCenterY(self.getHeight() / 2)
        self.characters[0].stop()

        self.player1 = self.characters[0]

        self.characters.append(Character())
        self.characters[1].color = Color(1, 0, 0, 1)
        self.canvas.add(self.characters[1].color)
        self.canvas.add(self.characters[1].rectangle)
        self.characters[1].setCenterX(400)
        self.characters[1].hasAI = True
        #Make the other character move toward the player:
        self.characters[1].targetX = self.characters[0].getCenterX()
        self.characters[1].targetY = self.characters[0].getCenterY()

        #Making update interval 1/60 sec make game run at 60 frames per second:
        Clock.schedule_interval(self.update, 1.0 / 60.0)

    def remove_character(self, thisguy):
        self.canvas.remove(thisguy.color)
        self.canvas.remove(thisguy.rectangle)
        self.characters.remove(thisguy)

    def generate_moving_character(self, pos, vec, pixels_per_frame):
        self.characters.append(Character())
        characterIndex = len(self.characters) - 1
        thisguy=self.characters[characterIndex]
        
        thisguy.color = Color(0, 1, 0, 1)
        self.canvas.add(thisguy.color)
        self.canvas.add(thisguy.rectangle)
        thisguy.set_center_pos(pos)
        thisguy.set_target_pos(pos[0] + vec[0] * 10000, pos[1] + vec[1] * 10000)
        thisguy.hasAI = False
        thisguy.maxSpeedPPS = pixels_per_frame
        return thisguy

    def generate_character(self, pos, vec, texture, rect):
        self.characters.append(Character())
        characterIndex = len(self.characters) - 1
        thisguy=self.characters[characterIndex]
        if texture is not None and rect is not None:
            #self.characters[characterIndex].rectangle.source = "ball.png"
            self.characters[characterIndex].rectangle.texture = texture
            u = rect.pos[0]/texture.size[0]
            v = rect.pos[1]/texture.size[1]
            y = v
            w = rect.size[0]/texture.size[0]
            h = rect.size[1]/texture.size[1]
            self.characters[characterIndex].rectangle.size = rect.size
            self.characters[characterIndex].rectangle.tex_coords = [u, v, u + w, v, u + w, y + h, u, y + h]
        else:
            self.characters[characterIndex].color = Color(0, 1, 0, 1)

        self.canvas.add(self.characters[characterIndex].color)
        self.canvas.add(self.characters[characterIndex].rectangle)
        self.characters[characterIndex].setCenterX(pos[0])
        self.characters[characterIndex].setCenterY(pos[1])
        self.characters[characterIndex].targetX = pos[0] + vec[0] * 1000
        self.characters[characterIndex].targetY = pos[1] + vec[1] * 1000
        self.characters[characterIndex].hasAI = False
        self.characters[characterIndex].maxSpeedPPS = 11
        thisguy.is_disposable=True
        print("created character " + str(characterIndex))

    def generate_character(self, pos, hasAI):
        self.characters.append(Character())
        characterIndex = len(self.characters) - 1
        thisguy=self.characters[characterIndex]
        
        thisguy.color = Color(0, 1, 0, 1)
        self.canvas.add(thisguy.color)
        self.canvas.add(thisguy.rectangle)
        thisguy.set_center_pos(pos)
        thisguy.stop()
        thisguy.hasAI = hasAI
        #thisguy.maxSpeedPPS = 3
        return thisguy

    def getWidth(self):
        return self.size[0]

    def getHeight(self):
        return self.size[1]

    def _keyboard_closed(self):
        print('Keyboard disconnected!')
        self._keyboard.unbind(on_key_down=self._on_keyboard_down)
        self._keyboard = None

    def update(self, dt):
        pass

    def _on_keyboard_down(self, keyboard, keycode, text, modifiers):
        print('The key' + str(keycode) + ' pressed')
        try:
            print(' - text is ' + str(text))
        except:
            print(' - text is not available')
        print(' - modifiers are '+ str(modifiers))
        # Keycode is composed of an integer + a string
        # If we hit escape, release the keyboard
        if keycode[1] == 'escape':
            keyboard.release()
        return True

    def canvasTouchDown(self, touch, *largs):
        touchX, touchY = largs[0].pos
        self.player1.targetX = touchX
        self.player1.targetY = touchY


class SquaresMovingApp(App):
    def build(self):
        return CanvasForm()

if __name__ == '__main__':
    SquaresMovingApp().run()
